@@ -10,10 +10,10 @@ Dataset <- R6::R6Class(
     finalize = function() {
 
       # Check the need for freeing handle
-      if (!lgb.is.null.handle(private$handle)) {
+      if (!lgb.is.null.handle(x = private$handle)) {
 
         # Freeing up handle
-        lgb.call("LGBM_DatasetFree_R", ret = NULL, private$handle)
+        lgb.call(fun_name = "LGBM_DatasetFree_R", ret = NULL, private$handle)
         private$handle <- NULL
 
       }
@@ -33,10 +33,10 @@ Dataset <- R6::R6Class(
                           ...) {
 
       # validate inputs early to avoid unnecessary computation
-      if (!(is.null(reference) || lgb.check.r6.class(reference, "lgb.Dataset"))) {
+      if (!(is.null(reference) || lgb.check.r6.class(object = reference, name = "lgb.Dataset"))) {
           stop("lgb.Dataset: If provided, reference must be a ", sQuote("lgb.Dataset"))
       }
-      if (!(is.null(predictor) || lgb.check.r6.class(predictor, "lgb.Predictor"))) {
+      if (!(is.null(predictor) || lgb.check.r6.class(object = predictor, name = "lgb.Predictor"))) {
           stop("lgb.Dataset: If provided, predictor must be a ", sQuote("lgb.Predictor"))
       }
 
@@ -113,7 +113,7 @@ Dataset <- R6::R6Class(
     construct = function() {
 
       # Check for handle null
-      if (!lgb.is.null.handle(private$handle)) {
+      if (!lgb.is.null.handle(x = private$handle)) {
         return(invisible(self))
       }
 
@@ -178,7 +178,7 @@ Dataset <- R6::R6Class(
       }
 
       # Generate parameter str
-      params_str <- lgb.params2str(private$params)
+      params_str <- lgb.params2str(params = private$params)
 
       # Get handle of reference dataset
       ref_handle <- NULL
@@ -194,9 +194,9 @@ Dataset <- R6::R6Class(
         if (is.character(private$raw_data)) {
 
           handle <- lgb.call(
-            "LGBM_DatasetCreateFromFile_R"
+            fun_name = "LGBM_DatasetCreateFromFile_R"
             , ret = handle
-            , lgb.c_str(private$raw_data)
+            , lgb.c_str(x = private$raw_data)
             , params_str
             , ref_handle
           )
@@ -205,7 +205,7 @@ Dataset <- R6::R6Class(
 
           # Are we using a matrix?
           handle <- lgb.call(
-            "LGBM_DatasetCreateFromMat_R"
+            fun_name = "LGBM_DatasetCreateFromMat_R"
             , ret = handle
             , private$raw_data
             , nrow(private$raw_data)
@@ -220,7 +220,7 @@ Dataset <- R6::R6Class(
           }
           # Are we using a dgCMatrix (sparsed matrix column compressed)
           handle <- lgb.call(
-            "LGBM_DatasetCreateFromCSC_R"
+            fun_name = "LGBM_DatasetCreateFromCSC_R"
             , ret = handle
             , private$raw_data@p
             , private$raw_data@i
@@ -251,7 +251,7 @@ Dataset <- R6::R6Class(
 
         # Construct subset
         handle <- lgb.call(
-          "LGBM_DatasetGetSubset_R"
+          fun_name = "LGBM_DatasetGetSubset_R"
           , ret = handle
           , ref_handle
           , c(private$used_indices) # Adding c() fixes issue in R v3.5
@@ -260,7 +260,7 @@ Dataset <- R6::R6Class(
         )
 
       }
-      if (lgb.is.null.handle(handle)) {
+      if (lgb.is.null.handle(x = handle)) {
         stop("lgb.Dataset.construct: cannot create Dataset handle")
       }
       # Setup class and private type
@@ -269,7 +269,7 @@ Dataset <- R6::R6Class(
 
       # Set feature names
       if (!is.null(private$colnames)) {
-        self$set_colnames(private$colnames)
+        self$set_colnames(colnames = private$colnames)
       }
 
       # Load init score if requested
@@ -277,7 +277,7 @@ Dataset <- R6::R6Class(
 
         # Setup initial scores
         init_score <- private$predictor$predict(
-          private$raw_data
+          data = private$raw_data
           , rawscore = TRUE
           , reshape = TRUE
         )
@@ -300,14 +300,14 @@ Dataset <- R6::R6Class(
         for (i in seq_along(private$info)) {
 
           p <- private$info[i]
-          self$setinfo(names(p), p[[1L]])
+          self$setinfo(name = names(p), info = p[[1L]])
 
         }
 
       }
 
       # Get label information existence
-      if (is.null(self$getinfo("label"))) {
+      if (is.null(self$getinfo(name = "label"))) {
         stop("lgb.Dataset.construct: label should be set")
       }
 
@@ -319,14 +319,24 @@ Dataset <- R6::R6Class(
     dim = function() {
 
       # Check for handle
-      if (!lgb.is.null.handle(private$handle)) {
+      if (!lgb.is.null.handle(x = private$handle)) {
 
         num_row <- 0L
         num_col <- 0L
 
         # Get numeric data and numeric features
-        c(lgb.call("LGBM_DatasetGetNumData_R", ret = num_row, private$handle),
-          lgb.call("LGBM_DatasetGetNumFeature_R", ret = num_col, private$handle))
+        c(
+          lgb.call(
+            fun_name = "LGBM_DatasetGetNumData_R"
+            , ret = num_row
+            , private$handle
+          ),
+          lgb.call(
+            fun_name = "LGBM_DatasetGetNumFeature_R"
+            , ret = num_col
+            , private$handle
+          )
+        )
 
       } else if (is.matrix(private$raw_data) || methods::is(private$raw_data, "dgCMatrix")) {
 
@@ -350,10 +360,13 @@ Dataset <- R6::R6Class(
     get_colnames = function() {
 
       # Check for handle
-      if (!lgb.is.null.handle(private$handle)) {
+      if (!lgb.is.null.handle(x = private$handle)) {
 
         # Get feature names and write them
-        cnames <- lgb.call.return.str("LGBM_DatasetGetFeatureNames_R", private$handle)
+        cnames <- lgb.call.return.str(
+            fun_name = "LGBM_DatasetGetFeatureNames_R"
+            , private$handle
+        )
         private$colnames <- as.character(base::strsplit(cnames, "\t")[[1L]])
         private$colnames
 
@@ -390,15 +403,15 @@ Dataset <- R6::R6Class(
 
       # Write column names
       private$colnames <- colnames
-      if (!lgb.is.null.handle(private$handle)) {
+      if (!lgb.is.null.handle(x = private$handle)) {
 
         # Merge names with tab separation
         merged_name <- paste0(as.list(private$colnames), collapse = "\t")
         lgb.call(
-          "LGBM_DatasetSetFeatureNames_R"
+          fun_name = "LGBM_DatasetSetFeatureNames_R"
           , ret = NULL
           , private$handle
-          , lgb.c_str(merged_name)
+          , lgb.c_str(x = merged_name)
         )
 
       }
@@ -421,17 +434,17 @@ Dataset <- R6::R6Class(
       # Check for info name and handle
       if (is.null(private$info[[name]])) {
 
-        if (lgb.is.null.handle(private$handle)) {
+        if (lgb.is.null.handle(x = private$handle)) {
           stop("Cannot perform getinfo before constructing Dataset.")
         }
 
         # Get field size of info
         info_len <- 0L
         info_len <- lgb.call(
-          "LGBM_DatasetGetFieldSize_R"
+          fun_name = "LGBM_DatasetGetFieldSize_R"
           , ret = info_len
           , private$handle
-          , lgb.c_str(name)
+          , lgb.c_str(x = name)
         )
 
         # Check if info is not empty
@@ -446,10 +459,10 @@ Dataset <- R6::R6Class(
           }
 
           ret <- lgb.call(
-            "LGBM_DatasetGetField_R"
+            fun_name = "LGBM_DatasetGetField_R"
             , ret = ret
             , private$handle
-            , lgb.c_str(name)
+            , lgb.c_str(x = name)
           )
 
           private$info[[name]] <- ret
@@ -482,15 +495,15 @@ Dataset <- R6::R6Class(
       # Store information privately
       private$info[[name]] <- info
 
-      if (!lgb.is.null.handle(private$handle) && !is.null(info)) {
+      if (!lgb.is.null.handle(x = private$handle) && !is.null(info)) {
 
         if (length(info) > 0L) {
 
           lgb.call(
-            "LGBM_DatasetSetField_R"
+            fun_name = "LGBM_DatasetSetField_R"
             , ret = NULL
             , private$handle
-            , lgb.c_str(name)
+            , lgb.c_str(x = name)
             , info
             , length(info)
           )
@@ -529,14 +542,14 @@ Dataset <- R6::R6Class(
       if (length(params) == 0L) {
         return(invisible(self))
       }
-      if (lgb.is.null.handle(private$handle)) {
+      if (lgb.is.null.handle(x = private$handle)) {
         private$params <- modifyList(private$params, params)
       } else {
         call_state <- 0L
         call_state <- .Call(
           "LGBM_DatasetUpdateParamChecking_R"
-          , lgb.params2str(private$params)
-          , lgb.params2str(params)
+          , lgb.params2str(params = private$params)
+          , lgb.params2str(params = params)
           , call_state
           , PACKAGE = "lightgbm"
         )
@@ -595,9 +608,9 @@ Dataset <- R6::R6Class(
     set_reference = function(reference) {
 
       # Set known references
-      self$set_categorical_feature(reference$.__enclos_env__$private$categorical_feature)
-      self$set_colnames(reference$get_colnames())
-      private$set_predictor(reference$.__enclos_env__$private$predictor)
+      self$set_categorical_feature(categorical_feature = reference$.__enclos_env__$private$categorical_feature)
+      self$set_colnames(colnames = reference$get_colnames())
+      private$set_predictor(predictor = reference$.__enclos_env__$private$predictor)
 
       # Check for identical references
       if (identical(private$reference, reference)) {
@@ -616,7 +629,7 @@ Dataset <- R6::R6Class(
       if (!is.null(reference)) {
 
         # Reference is unknown
-        if (!lgb.check.r6.class(reference, "lgb.Dataset")) {
+        if (!lgb.check.r6.class(object = reference, name = "lgb.Dataset")) {
           stop("set_reference: Can only use lgb.Dataset as a reference")
         }
 
@@ -637,10 +650,10 @@ Dataset <- R6::R6Class(
       # Store binary data
       self$construct()
       lgb.call(
-        "LGBM_DatasetSaveBinary_R"
+        fun_name = "LGBM_DatasetSaveBinary_R"
         , ret = NULL
         , private$handle
-        , lgb.c_str(fname)
+        , lgb.c_str(x = fname)
       )
       return(invisible(self))
     }
@@ -663,7 +676,7 @@ Dataset <- R6::R6Class(
     get_handle = function() {
 
       # Get handle and construct if needed
-      if (lgb.is.null.handle(private$handle)) {
+      if (lgb.is.null.handle(x = private$handle)) {
         self$construct()
       }
       private$handle
@@ -687,7 +700,7 @@ Dataset <- R6::R6Class(
       if (!is.null(predictor)) {
 
         # Predictor is unknown
-        if (!lgb.check.r6.class(predictor, "lgb.Predictor")) {
+        if (!lgb.check.r6.class(object = predictor, name = "lgb.Predictor")) {
           stop("set_predictor: Can only use lgb.Predictor as predictor")
         }
 
@@ -778,12 +791,12 @@ lgb.Dataset <- function(data,
 lgb.Dataset.create.valid <- function(dataset, data, info = list(), ...) {
 
   # Check if dataset is not a dataset
-  if (!lgb.is.Dataset(dataset)) {
+  if (!lgb.is.Dataset(x = dataset)) {
     stop("lgb.Dataset.create.valid: input data should be an lgb.Dataset object")
   }
 
   # Create validation dataset
-  invisible(dataset$create_valid(data, info, ...))
+  invisible(dataset$create_valid(data = data, info = info, ...))
 
 }
 
@@ -804,7 +817,7 @@ lgb.Dataset.create.valid <- function(dataset, data, info = list(), ...) {
 lgb.Dataset.construct <- function(dataset) {
 
   # Check if dataset is not a dataset
-  if (!lgb.is.Dataset(dataset)) {
+  if (!lgb.is.Dataset(x = dataset)) {
     stop("lgb.Dataset.construct: input data should be an lgb.Dataset object")
   }
 
@@ -839,7 +852,7 @@ lgb.Dataset.construct <- function(dataset) {
 dim.lgb.Dataset <- function(x, ...) {
 
   # Check if dataset is not a dataset
-  if (!lgb.is.Dataset(x)) {
+  if (!lgb.is.Dataset(x = x)) {
     stop("dim.lgb.Dataset: input data should be an lgb.Dataset object")
   }
 
@@ -875,7 +888,7 @@ dim.lgb.Dataset <- function(x, ...) {
 dimnames.lgb.Dataset <- function(x) {
 
   # Check if dataset is not a dataset
-  if (!lgb.is.Dataset(x)) {
+  if (!lgb.is.Dataset(x = x)) {
     stop("dimnames.lgb.Dataset: input data should be an lgb.Dataset object")
   }
 
@@ -901,7 +914,7 @@ dimnames.lgb.Dataset <- function(x) {
 
   if (is.null(value[[2L]])) {
 
-    x$set_colnames(NULL)
+    x$set_colnames(colnames = NULL)
     return(x)
 
   }
@@ -918,7 +931,7 @@ dimnames.lgb.Dataset <- function(x) {
   }
 
   # Set column names properly, and return
-  x$set_colnames(value[[2L]])
+  x$set_colnames(colnames = value[[2L]])
   x
 
 }
@@ -952,12 +965,12 @@ slice <- function(dataset, ...) {
 slice.lgb.Dataset <- function(dataset, idxset, ...) {
 
   # Check if dataset is not a dataset
-  if (!lgb.is.Dataset(dataset)) {
+  if (!lgb.is.Dataset(x = dataset)) {
     stop("slice.lgb.Dataset: input dataset should be an lgb.Dataset object")
   }
 
   # Return sliced set
-  invisible(dataset$slice(idxset, ...))
+  invisible(dataset$slice(idxset = idxset, ...))
 
 }
 
@@ -1003,11 +1016,11 @@ getinfo <- function(dataset, ...) {
 getinfo.lgb.Dataset <- function(dataset, name, ...) {
 
   # Check if dataset is not a dataset
-  if (!lgb.is.Dataset(dataset)) {
+  if (!lgb.is.Dataset(x = dataset)) {
     stop("getinfo.lgb.Dataset: input dataset should be an lgb.Dataset object")
   }
 
-  dataset$getinfo(name)
+  dataset$getinfo(name = name)
 
 }
 
@@ -1056,12 +1069,12 @@ setinfo <- function(dataset, ...) {
 #' @export
 setinfo.lgb.Dataset <- function(dataset, name, info, ...) {
 
-  if (!lgb.is.Dataset(dataset)) {
+  if (!lgb.is.Dataset(x = dataset)) {
     stop("setinfo.lgb.Dataset: input dataset should be an lgb.Dataset object")
   }
 
   # Set information
-  invisible(dataset$setinfo(name, info))
+  invisible(dataset$setinfo(name = name, info = info))
 }
 
 #' @name lgb.Dataset.set.categorical
@@ -1088,12 +1101,12 @@ setinfo.lgb.Dataset <- function(dataset, name, info, ...) {
 #' @export
 lgb.Dataset.set.categorical <- function(dataset, categorical_feature) {
 
-  if (!lgb.is.Dataset(dataset)) {
+  if (!lgb.is.Dataset(x = dataset)) {
     stop("lgb.Dataset.set.categorical: input dataset should be an lgb.Dataset object")
   }
 
   # Set categoricals
-  invisible(dataset$set_categorical_feature(categorical_feature))
+  invisible(dataset$set_categorical_feature(categorical_feature = categorical_feature))
 
 }
 
@@ -1120,12 +1133,12 @@ lgb.Dataset.set.categorical <- function(dataset, categorical_feature) {
 lgb.Dataset.set.reference <- function(dataset, reference) {
 
   # Check if dataset is not a dataset
-  if (!lgb.is.Dataset(dataset)) {
+  if (!lgb.is.Dataset(x = dataset)) {
     stop("lgb.Dataset.set.reference: input dataset should be an lgb.Dataset object")
   }
 
   # Set reference
-  invisible(dataset$set_reference(reference))
+  invisible(dataset$set_reference(reference = reference))
 }
 
 #' @name lgb.Dataset.save
@@ -1148,7 +1161,7 @@ lgb.Dataset.set.reference <- function(dataset, reference) {
 lgb.Dataset.save <- function(dataset, fname) {
 
   # Check if dataset is not a dataset
-  if (!lgb.is.Dataset(dataset)) {
+  if (!lgb.is.Dataset(x = dataset)) {
     stop("lgb.Dataset.set: input dataset should be an lgb.Dataset object")
   }
 
@@ -1158,5 +1171,5 @@ lgb.Dataset.save <- function(dataset, fname) {
   }
 
   # Store binary
-  invisible(dataset$save_binary(fname))
+  invisible(dataset$save_binary(fname = fname))
 }
