@@ -121,7 +121,7 @@ lightgbm <- function(data,
   dtrain <- data
 
   # Check whether data is lgb.Dataset, if not then create lgb.Dataset manually
-  if (!lgb.is.Dataset(dtrain)) {
+  if (!lgb.is.Dataset(x = dtrain)) {
     dtrain <- lgb.Dataset(data = data, label = label, weight = weight)
   }
 
