@@ -6,23 +6,37 @@
 
 #include <iostream>
 
+#include "network/linkers.h"
+
 int main(int argc, char** argv) {
+  bool success = false;
   try {
     LightGBM::Application app(argc, argv);
     app.Run();
+
+#ifdef USE_MPI
+    LightGBM::Linkers::MpiFinalizeIfIsParallel();
+#endif
+
+    success = true;
   }
   catch (const std::exception& ex) {
     std::cerr << "Met Exceptions:" << std::endl;
     std::cerr << ex.what() << std::endl;
-    exit(-1);
   }
   catch (const std::string& ex) {
     std::cerr << "Met Exceptions:" << std::endl;
     std::cerr << ex << std::endl;
-    exit(-1);
   }
   catch (...) {
     std::cerr << "Unknown Exceptions" << std::endl;
+  }
+
+  if (!success) {
+#ifdef USE_MPI
+    LightGBM::Linkers::MpiAbortIfIsParallel();
+#endif
+
     exit(-1);
   }
 }
